var strServer = "";
var strDatabaseName = "ultrasound";
var bitsClear = 0;
var bitsSet = 0;
var fAutoVerify = false;

// ParameterDirection enum
var adParamOutput = 2;
var adParamReturnValue = 4;

// CommandType enum
var adCmdStoredProc = 4;

// DataType enum
var adInteger = 3;

var rgCollectionBits =
[
"CollectConnectionSchedule",    // 1
"CollectFilereference",         // 2
"CollectFileoutlog",            // 4
"CollectEventLog",              // 8
"CollectVolatileConnections"    // 16
];

// Default is to collect all exception VolatileConnections.
var CollectionDefaults = 0x1F;  // NOTE: Should be kept in sync with the CollectionDefaults in SuppressionFlags.h

//---------------------------------------------------------------------
// Displays usage information
function Usage()
{
    var i;
    WScript.Echo("Display and set Ultrasound data collection Collection options");
    WScript.Echo("Usage:");
    WScript.Echo("  [-?]           show usage information")
    WScript.Echo("   -s server     SQL server")
    WScript.Echo("  [-d database]  database name, default to 'Ultrasound'")
    for (i = 0; i < rgCollectionBits.length; ++i)
    {
        WScript.Echo("  [+/- " + rgCollectionBits[i] + "]");
    }
    WScript.Echo("\nExamples:");
    WScript.Echo("");
    WScript.Echo("To Display current settings.");
    WScript.Echo("cscript SetUltrasoundCollectionOptions.js -s SQL\\FRSMon -d MyUltrasoundDB");
    WScript.Echo("");
    WScript.Echo("To Disable collection schedule and volatile connections data collection.");
    WScript.Echo("cscript SetUltrasoundCollectionOptions.js -s SQL\\FRSMon -d MyUltrasoundDB -CollectConnectionSchedule -CollectVolatileConnections");
    WScript.Echo("");
    WScript.Echo("To Enable volatile connection data collection.");
    WScript.Echo("cscript SetUltrasoundColletionOptions.js -s SQL\\FRSMon -d MyUltrasoundDB +CollectVolatileConnections");
    WScript.Echo("");

}

function CheckArgBit(strArg)
{
    var str = strArg.substr(1);
    var i;
    for (i = 0; i < rgCollectionBits.length; ++i)
    {
        if (str == rgCollectionBits[i].toLowerCase())
        {
            if (strArg.charAt(0) == '-')
                bitsClear |= (1 << i);
            else
                bitsSet |= (1 << i);

            return true;
        }
    }
    return false;
}

//---------------------------------------------------------------------
// parse command line
function ParseCommandLine()
{
    var enumArgs = new Enumerator(WScript.Arguments);
    for (; !enumArgs.atEnd(); enumArgs.moveNext())
    {
        var arg = enumArgs.item().toLowerCase();

        if (arg == "-?" || arg == "/?" || arg == "?")
        {
            Usage();
            WScript.Quit(1);
        }
        else if (arg == "-s")
        {
            enumArgs.moveNext();
            strServer = enumArgs.item();
        }
        else if (arg == "-d")
        {
            enumArgs.moveNext();
            strDatabaseName = enumArgs.item();
        }
        else if (arg == "-y")
        {
            fAutoVerify = true;
        }
        else
        {
            if (!((arg.charAt(0) == '-' || arg.charAt(0) == '+') && CheckArgBit(arg)))
            {
                Usage();
                WScript.Quit(0);
            }
        }
    }

    // validate the input
    if (strServer == "")
    {
        Usage();
        WScript.Quit(0);
    }
}

main();

function BitChangedToOn(oldFlags, flags, strFlagName)
{
    var i;
    for (i = 0; i < rgCollectionBits.length; ++i)
    {
        if (strFlagName.toLowerCase() == rgCollectionBits[i].toLowerCase())
        {
            var flagMask = (1 << i);

            return ((oldFlags & flagMask) == 0) && ((flags & flagMask) != 0);
        }
    }
    WScript.Echo("ASSERT: Unknown flag name " + strFlagName);
    return false;
}

function BitChanged(oldFlags, flags, strFlagName)
{
    var i;
    for (i = 0; i < rgCollectionBits.length; ++i)
    {
        if (strFlagName.toLowerCase() == rgCollectionBits[i].toLowerCase())
        {
            var flagMask = (1 << i);

            return (oldFlags & flagMask) ^ (flags & flagMask);
        }
    }
    WScript.Echo("ASSERT: Unknown flag name " + strFlagName);
    return false;
}

function main()
{
    ParseCommandLine();
    var strConnect =
    "Provider=SQLOLEDB;" +
    "Data Source=" + strServer+ ";" +
    "Integrated Security=SSPI;" +
    "Initial Catalog=" + strDatabaseName + ";" +
    "Persist Security Info=False;";

    var oConnect = new ActiveXObject("ADODB.Connection");
    oConnect.Open(strConnect);

    CheckVersion(oConnect);

    var flags = GetCollectionFlags(oConnect);
    WScript.Echo("Current Collection options: " + ShowBits(flags, rgCollectionBits));

    if (bitsSet || bitsClear)
    {
        var oldFlags = flags;
        flags |= bitsSet;
        flags &= ~bitsClear;
        // Detect interesting changes...
        // Due to race conditions that can happen
        // if this script makes changes to the DB while the controller is
        // active, we warn the user that they should shutdown the controller.
        if (BitChangedToOn(oldFlags, flags, "CollectFilereference") || BitChangedToOn(oldFlags, flags, "CollectFileoutlog") || BitChanged(oldFlags, flags, "CollectConnectionSchedule"))
        {
            var yes = Verify("\nThe Ultrasound Controller service must be stopped before performing this\n" +
                             "operation. Have you stopped the controller");

            if (!yes)
            {
                WScript.Echo("\nStop the controller then try again later.");
                WScript.Quit(1);
            }
            if (BitChangedToOn(oldFlags, flags, "CollectFilereference") || BitChangedToOn(oldFlags, flags, "CollectFileoutlog"))
            {
                ClearOutlogSeqNum(oConnect);
            }
            if (BitChanged(oldFlags, flags, "CollectConnectionSchedule"))
            {
                ClearConnectionChanged(oConnect);
            }
        }

        SetCollectionFlags(oConnect, flags);
        WScript.Echo("New Collection options: " + ShowBits(flags, rgCollectionBits));
    }

    WScript.Quit(0);
}

function GetCollectionFlagsField(oConnect, collectionIndex)
{
    var oCommand = new ActiveXObject("ADODB.Command");
    var oRS;

    oCommand.ActiveConnection = oConnect;
    oCommand.CommandText = "Select Value from PropertyBag Where Parameter = N'" + rgCollectionBits[collectionIndex] + "'";

    oRS = oCommand.Execute();

    if (oRS.EOF)
    {
        return null;
    }

    var oField = oRS.Fields.Item("Value");
    return oField;
}

function GetCollectionFlags(oConnect)
{
    var i;
    var oField;
    var flags = CollectionDefaults;

    for (i = 0; i < rgCollectionBits.length; ++i)
    {
        var oField = GetCollectionFlagsField(oConnect, i);
        if (oField)
        {
            if (oField.value != 0)
            {
                flags |= (1 << i);
            }
            else
            {
                flags &= ~(1 << i);
            }
        }
    }
    return flags;
}

function SetCollectionFlags(oConnect, flags)
{
    var oCommand = new ActiveXObject("ADODB.Command");
    var oRS;

    oCommand.ActiveConnection = oConnect;

    for (i = 0; i < rgCollectionBits.length; ++i)
    {
        var value = (flags & (1 << i)) ? 1 : 0;
        var strUpdate = "Update PropertyBag Set Value=" + value + " where Parameter = N'" + rgCollectionBits[i] + "'";
        var strInsert = "Insert into PropertyBag (Parameter, Value) VALUES('" + rgCollectionBits[i] + "', " +  value + ")";

        var oField = GetCollectionFlagsField(oConnect, i);
        if (oField)
            oCommand.CommandText = strUpdate;
        else
            oCommand.CommandText = strInsert;

        oRS = oCommand.Execute();
    }
}

function ShowBits(value, rgBitStrings)
{
    var i = 0;
    var strBits = "";
    while (i < rgBitStrings.length)
    {
        strBits += "\n\t";
        if (value & 1)
            strBits +=  "+" + rgBitStrings[i]
        else
            strBits +=  "-" + rgBitStrings[i]

        value = (value >> 1);
        ++i;
    }
    return strBits;
}

function ClearOutlogSeqNum(oConnect)
{
    var oCommand = new ActiveXObject("ADODB.Command");
    var strCmdText =
    "update MemberData set OutlogSeq=0 " +
    "where UpdateID in ( " +
    "    Select memberdata.UpdateID from memberdata " +
    "    join UpdateList UL on memberdata.UpdateID = UL.UpdateID " +
    "    where UL.NextSuccessfulUpdateID is null " +
    " )";

    oCommand.ActiveConnection = oConnect;
    oCommand.CommandText = strCmdText;
    oCommand.Execute();
}

function ClearConnectionChanged(oConnect)
{
    var oCommand = new ActiveXObject("ADODB.Command");
    var strCmdText = "update connection set changed = dateadd(second, -1, changed)";

    oCommand.ActiveConnection = oConnect;
    oCommand.CommandText = strCmdText;
    oCommand.Execute();
}

function Verify(strPrompt)
{
    if (fAutoVerify)
        return true;

    var result;
    do
    {
        WScript.StdOut.Write(strPrompt + " Y/N?");
        result = WScript.StdIn.ReadLine();

        if (result > "")
            result = result.toLowerCase();

        if (result == "y" || result == "yes")
            return true;
    }
    while (result != "n" && result != "no");
    return false;
}

function CheckVersion(oConnect)
{
    var oCommand = new ActiveXObject("ADODB.Command");
    oCommand.ActiveConnection = oConnect;
    oCommand.CommandText = "dbo.GetDBControllerVersion";
    oCommand.CommandType = adCmdStoredProc;
    oCommand.NamedParameters = true;

    var oRetVal = oCommand.CreateParameter("retval", adInteger, adParamReturnValue);
    oCommand.Parameters.Append(oRetVal);

    var oDBMajor    = AddSPParameter(oCommand, "@DBMajor",    adInteger, adParamOutput);
    var oDBMinor    = AddSPParameter(oCommand, "@DBMinor",    adInteger, adParamOutput);
    var oDBBuild    = AddSPParameter(oCommand, "@DBBuild",    adInteger, adParamOutput);
    var oDBRevision = AddSPParameter(oCommand, "@DBRevision", adInteger, adParamOutput);
    var oMajor    = AddSPParameter(oCommand, "@DBRequiresControllerMajor",    adInteger, adParamOutput);
    var oMinor    = AddSPParameter(oCommand, "@DBRequiresControllerMinor",    adInteger, adParamOutput);
    var oBuild    = AddSPParameter(oCommand, "@DBRequiresControllerBuild",    adInteger, adParamOutput);
    var oRevision = AddSPParameter(oCommand, "@DBRequiresControllerRevision", adInteger, adParamOutput);

    oCommand.Execute();

    var DBVer = [oDBMajor, oDBMinor, oDBBuild, oDBRevision];
    var ControllerVer = [oMajor, oMinor, oBuild, oRevision];
    var RequiredVer = [1, 0, 3287, 0];

    if (!VersionCompare(RequiredVer, DBVer))
    {
        WScript.Echo("Database version is older than expected.");
        WScript.Echo("Database version is " + DBVer.join("."));
        WScript.Echo("Required version is " + RequiredVer.join("."));
        WScript.Quit(1);
    }
}

//---------------------------------------------------------------------
// add SP parameter
function AddSPParameter(command, name, type, direction, len, value)
{
    var oParam = command.CreateParameter(name,
                                         type,
                                         direction,
                                         len,
                                         value);
    command.Parameters.Append(oParam);
    return oParam;
}

function VersionCompare(verExpected, verActual)
{
    var i;

    for(i = 0; i < verExpected.length; ++i)
    {
        if (verExpected[i] > verActual[i])
        {
            return false;
        }
        if (verExpected[i] < verActual[i])
        {
            return true;
        }
    }
    return true;
}
